"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ragie import utils
from ragie.types import BaseModel


class ErrorMessageErrorData(BaseModel):
    detail: str


class ErrorMessageError(Exception):
    data: ErrorMessageErrorData

    def __init__(self, data: ErrorMessageErrorData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, ErrorMessageErrorData)
