"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ragie.types import BaseModel
from typing import Any, Dict
from typing_extensions import TypedDict


class ScoredChunkTypedDict(TypedDict):
    text: str
    score: float
    document_id: str
    document_metadata: Dict[str, Any]


class ScoredChunk(BaseModel):
    text: str

    score: float

    document_id: str

    document_metadata: Dict[str, Any]
