"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .updatedocumentrawparams import (
    UpdateDocumentRawParams,
    UpdateDocumentRawParamsTypedDict,
)
from ragie.types import BaseModel
from ragie.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class UpdateDocumentRawRequestTypedDict(TypedDict):
    document_id: str
    r"""The id of the document."""
    update_document_raw_params: UpdateDocumentRawParamsTypedDict


class UpdateDocumentRawRequest(BaseModel):
    document_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The id of the document."""

    update_document_raw_params: Annotated[
        UpdateDocumentRawParams,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
